/// <reference types="node" />
import { Colors, CommandMetadata, CommandMetadataInput, CommandMetadataOption, ICommand, INamespace, NamespaceLocateResult } from '../definitions';
import { Command, Namespace } from './command';
export declare class BaseExecutorDeps<C extends ICommand<C, N, M, I, O>, N extends INamespace<C, N, M, I, O>, M extends CommandMetadata<I, O>, I extends CommandMetadataInput, O extends CommandMetadataOption> {
    readonly namespace: N;
    readonly colors?: Colors;
    readonly stdout?: NodeJS.WriteStream;
    readonly stderr?: NodeJS.WriteStream;
}
export declare class BaseExecutor<C extends ICommand<C, N, M, I, O>, N extends INamespace<C, N, M, I, O>, M extends CommandMetadata<I, O>, I extends CommandMetadataInput, O extends CommandMetadataOption> {
    readonly colors: Colors;
    readonly namespace: N;
    readonly stdout: NodeJS.WriteStream;
    readonly stderr: NodeJS.WriteStream;
    constructor({namespace, stdout, stderr, colors}: BaseExecutorDeps<C, N, M, I, O>);
    execute(argv: string[], env: {
        [key: string]: string;
    }): Promise<void>;
    run(command: C, cmdargs: string[], options?: {
        location?: NamespaceLocateResult<C, N, M, I, O>;
        env?: {
            [key: string]: string;
        };
    }): Promise<void>;
    formatHelp(location: NamespaceLocateResult<C, N, M, I, O>): Promise<string>;
}
export declare class Executor extends BaseExecutor<Command, Namespace, CommandMetadata, CommandMetadataInput, CommandMetadataOption> {
}
export declare function execute<C extends ICommand<C, N, M, I, O>, N extends INamespace<C, N, M, I, O>, M extends CommandMetadata<I, O>, I extends CommandMetadataInput, O extends CommandMetadataOption>({namespace, argv, env, ...rest}: {
    namespace: N;
    argv: string[];
    env: {
        [key: string]: string;
    };
} & Partial<BaseExecutorDeps<C, N, M, I, O>>): Promise<void>;
