"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const lodash = require("lodash");
const guards_1 = require("../guards");
const colors_1 = require("./colors");
const help_1 = require("./help");
const options_1 = require("./options");
const guards_2 = require("../guards");
class BaseExecutorDeps {
}
exports.BaseExecutorDeps = BaseExecutorDeps;
class BaseExecutor {
    constructor({ namespace, stdout, stderr, colors }) {
        this.namespace = namespace;
        this.colors = colors ? colors : colors_1.DEFAULT_COLORS;
        this.stdout = stdout ? stdout : process.stdout;
        this.stderr = stderr ? stderr : process.stderr;
    }
    execute(argv, env) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const parsedArgs = options_1.stripOptions(argv, { includeSeparated: false });
            const location = yield this.namespace.locate(parsedArgs);
            if (argv.includes('--help') || guards_2.isNamespace(location.obj)) {
                this.stdout.write(yield this.formatHelp(location));
            }
            else {
                const cmd = location.obj;
                const cmdargs = lodash.drop(argv, location.path.length - 1);
                yield this.run(cmd, cmdargs, { location, env });
            }
        });
    }
    run(command, cmdargs, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const metadata = yield command.getMetadata();
            const cmdoptions = options_1.parseArgs(cmdargs, options_1.metadataToParseArgsOptions(metadata));
            const cmdinputs = cmdoptions._;
            yield command.validate(cmdinputs);
            yield command.run(cmdinputs, cmdoptions, options);
        });
    }
    formatHelp(location) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (guards_1.isCommand(location.obj)) {
                const formatter = new help_1.CommandHelpFormatter({ location, command: location.obj });
                return formatter.format();
            }
            else {
                const formatter = new help_1.NamespaceHelpFormatter({ location, namespace: location.obj });
                return formatter.format();
            }
        });
    }
}
exports.BaseExecutor = BaseExecutor;
class Executor extends BaseExecutor {
}
exports.Executor = Executor;
function execute(_a) {
    var { namespace, argv, env } = _a, rest = tslib_1.__rest(_a, ["namespace", "argv", "env"]);
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const executor = new BaseExecutor(Object.assign({ namespace }, rest));
        yield executor.execute(argv, env);
    });
}
exports.execute = execute;
