import { Colors, CommandMetadata, CommandMetadataInput, CommandMetadataOption, HydratedCommandMetadata, ICommand, INamespace, NamespaceLocateResult, NamespaceMetadata } from '../definitions';
export declare abstract class HelpFormatter {
    protected readonly colors: Colors;
    constructor({colors}: {
        colors?: Colors;
    });
    abstract format(): Promise<string>;
}
export interface NamespaceHelpFormatterDeps<C extends ICommand<C, N, M, I, O>, N extends INamespace<C, N, M, I, O>, M extends CommandMetadata<I, O>, I extends CommandMetadataInput, O extends CommandMetadataOption> {
    readonly location: NamespaceLocateResult<C, N, M, I, O>;
    readonly namespace: N;
    readonly colors?: Colors;
}
export declare class NamespaceHelpFormatter<C extends ICommand<C, N, M, I, O>, N extends INamespace<C, N, M, I, O>, M extends CommandMetadata<I, O>, I extends CommandMetadataInput, O extends CommandMetadataOption> extends HelpFormatter {
    protected readonly location: NamespaceLocateResult<C, N, M, I, O>;
    protected readonly namespace: N;
    protected readonly dotswidth: number;
    protected _metadata: NamespaceMetadata;
    protected _fullName: string;
    constructor({location, namespace, colors}: NamespaceHelpFormatterDeps<C, N, M, I, O>);
    filterCommandCallback?(cmd: M & HydratedCommandMetadata<C, N, M, I, O>): Promise<boolean>;
    getNamespaceMetadata(): Promise<NamespaceMetadata>;
    getNamespaceFullName(): Promise<string>;
    formatHeader(): Promise<string>;
    formatDescription(): Promise<string>;
    formatSubtitle(): Promise<string>;
    formatLongDescription(): Promise<string>;
    getExtraOptions(): Promise<string[]>;
    formatUsage(): Promise<string>;
    formatCommands(): Promise<string>;
    formatCommandGroup(title: string, commands: ReadonlyArray<M & HydratedCommandMetadata<C, N, M, I, O>>): Promise<string>;
    getListOfCommandDetails(commands: ReadonlyArray<M & HydratedCommandMetadata<C, N, M, I, O>>): Promise<string[]>;
    getListOfNamespaceDetails(commands: ReadonlyArray<M & HydratedCommandMetadata<C, N, M, I, O>>): Promise<string[]>;
    formatBeforeNamespaceDescription(meta: NamespaceMetadata): Promise<string>;
    formatBeforeCommandDescription(cmd: M & HydratedCommandMetadata<C, N, M, I, O>): Promise<string>;
    formatAfterCommandDescription(cmd: M & HydratedCommandMetadata<C, N, M, I, O>): Promise<string>;
    format(): Promise<string>;
}
export interface CommandHelpFormatterDeps<C extends ICommand<C, N, M, I, O>, N extends INamespace<C, N, M, I, O>, M extends CommandMetadata<I, O>, I extends CommandMetadataInput, O extends CommandMetadataOption> {
    readonly location: NamespaceLocateResult<C, N, M, I, O>;
    readonly command: C;
    readonly colors?: Colors;
}
export declare class CommandHelpFormatter<C extends ICommand<C, N, M, I, O>, N extends INamespace<C, N, M, I, O>, M extends CommandMetadata<I, O>, I extends CommandMetadataInput, O extends CommandMetadataOption> extends HelpFormatter {
    protected readonly location: NamespaceLocateResult<C, N, M, I, O>;
    protected readonly command: C;
    protected readonly dotswidth: number;
    protected _metadata: M;
    protected _fullName: string;
    constructor({location, command, colors}: CommandHelpFormatterDeps<C, N, M, I, O>);
    getCommandMetadata(): Promise<M>;
    getCommandFullName(): Promise<string>;
    filterOptionCallback(options: O): boolean;
    formatHeader(): Promise<string>;
    formatDescription(): Promise<string>;
    formatSubtitle(): Promise<string>;
    formatLongDescription(): Promise<string>;
    formatInlineInput(input: I): Promise<string>;
    formatUsage(): Promise<string>;
    formatInputs(): Promise<string>;
    formatOptionLine(opt: O): Promise<string>;
    formatBeforeOptionDescription(opt: O): Promise<string>;
    formatAfterOptionDescription(opt: O): Promise<string>;
    formatOptionDefault(opt: O): Promise<string>;
    formatOptions(): Promise<string>;
    formatOptionsGroup(title: string, options: O[]): Promise<string>;
    formatExamples(): Promise<string>;
    format(): Promise<string>;
}
