"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const fs = require("fs");
const path = require("path");
const guards_1 = require("../guards");
const fs_1 = require("./fs");
const path_1 = require("./path");
exports.ERROR_INVALID_PACKAGE_JSON = 'INVALID_PACKAGE_JSON';
/**
 * Lightweight version of https://github.com/npm/validate-npm-package-name
 */
function isValidPackageName(name) {
    return encodeURIComponent(name) === name;
}
exports.isValidPackageName = isValidPackageName;
function readPackageJsonFile(p) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const packageJson = yield fs_1.fsReadJsonFile(p);
        if (!guards_1.isPackageJson(packageJson)) {
            throw exports.ERROR_INVALID_PACKAGE_JSON;
        }
        return packageJson;
    });
}
exports.readPackageJsonFile = readPackageJsonFile;
function compileNodeModulesPaths(filePath) {
    return path_1.compilePaths(filePath).map(f => path.join(f, 'node_modules'));
}
exports.compileNodeModulesPaths = compileNodeModulesPaths;
/**
 * Poorly implemented shim for Node 8+ `require.resolve()`, with `paths`
 * option.
 *
 * @see https://nodejs.org/docs/latest-v8.x/api/modules.html#modules_require_resolve_request_options
 * @see https://nodejs.org/docs/latest-v8.x/api/modules.html#modules_all_together
 */
function resolve(pkg, options) {
    const paths = options && options.paths ? options.paths : undefined;
    if (!paths) {
        // There is absolutely no reason to use this shim without the `paths`
        // option--use the built-in resolver.
        return require.resolve(pkg);
    }
    const safeStat = (filePath) => {
        try {
            return fs.statSync(filePath);
        }
        catch (e) {
            // ignore
        }
    };
    for (const p of paths) {
        const filePath = path.join(p, pkg);
        const filePaths = [filePath, filePath + '.js', filePath + '.json'];
        const stats = filePaths.map((p) => [p, safeStat(p)]);
        const found = stats.find(([p, s]) => typeof s !== 'undefined' && s.isFile());
        if (found) {
            return found[0];
        }
        const [[, stat]] = stats;
        if (stat && stat.isDirectory()) {
            try {
                const packageJson = JSON.parse(fs.readFileSync(path.join(filePath, 'package.json'), { encoding: 'utf8' }));
                if (packageJson.main) {
                    const mainPath = path.join(filePath, packageJson.main);
                    const mainStat = safeStat(mainPath);
                    if (mainStat && mainStat.isFile()) {
                        return mainPath;
                    }
                }
            }
            catch (e) {
                // ignore
            }
        }
    }
    const err = new Error(`Cannot find module '${pkg}'`);
    err.code = 'MODULE_NOT_FOUND';
    throw err;
}
exports.resolve = resolve;
